<?php

/**
 * Filename: class-xenword-login-xfusers.php
 *
 * Creates the login of a XenForo user into WordPress.
 * Optionally calls a function to copy the XF user over to WP database.
 *
 * @author      LPH
 * @version     2.5.0.04
 * @package     XenWord
 * @subpackage  includes
 *
 * @since 2.1.0.03
 *
 * @since 2.5.0.04
 * Conditional to call xenword_add_wp_user() only if user is not in WP database
 *
 * @since 2.5.1.03
 *        Added apply_filter to match WP get_currentuserinfo()
 * @since 2.5.1.04
 *        Surrounded $wpdb->users with {}
 *        Added missing return in get_currentuserinfo()
 */

/**
 * This section is for security. Do not modify this part:
 * @ignore
 */

if ( ! defined( 'ABSPATH' ) ) exit;

if ( ! function_exists( 'wp_get_current_user' ) ) :

	/**
	 * This replacement function will no longer work after WordPress 4.5
	 * The pluggable function was deprecated in WP 4.5
	 *
	 * @return void|boolean
	 *
	 * @since 2.5.1.03
	 *        Added apply_filter to match WP get_currentuserinfo()
	 *
	 * @since 3.0.2.01
	 *        wp_get_current_user instead of get_currentuserinfo()
	 */

	function wp_get_current_user() {
		global $current_user;

		if ( ! empty( $current_user ) ) {
			if ( $current_user instanceof WP_User ) {
				return $current_user;
			}

			// Upgrade stdClass to WP_User
			if ( is_object( $current_user ) && isset( $current_user->ID ) ) {
				$cur_id       = $current_user->ID;
				$current_user = null;
				wp_set_current_user( $cur_id );
				return $current_user;
			}

			// $current_user has a junk value. Force to WP_User with ID 0.
			$current_user = null;
			wp_set_current_user( 0 );
			return $current_user;
		}

		if ( defined( 'XMLRPC_REQUEST' ) && XMLRPC_REQUEST ) {
			wp_set_current_user( 0 );
			return $current_user;
		}

		$visitor = XenWord::getVisitor();

		$user_id = $visitor['user_id'];

		// Conditional for no XenForo user is logged in
		if ( $user_id == 0 ) {
			$current_user = null;
			wp_set_current_user( 0 );
			return $current_user;
		}

		/**
		 * Filter the current user.
		 *
		 * The default filters use this to determine the current user from the
		 * request's cookies, if available.
		 *
		 * Returning a value of false will effectively short-circuit setting
		 * the current user.
		 *
		 * @since 3.9.0
		 *
		 * @param int|bool $user_id User ID if one has been determined, false otherwise.
		 */
		$user_id = apply_filters( 'determine_current_user', false );
		if ( ! $user_id ) {
			wp_set_current_user( 0 );
			return $current_user;
		}

		$current_user = get_userdata( $user_id );

		wp_set_current_user( $user_id );

		wp_validate_auth_cookie($cookie = '', $scheme = '');

		// Check to determine if adding XF users to WP database
		XenWord_XF_Users::check_options();

		return $current_user;
	}

endif;

if ( ! function_exists( 'get_userdata' ) ) :

	/**
	 * Retrieve user info by $user_id
	 *
	 * @param $user_id
	 *
	 * @return boolean|WP_User
	 */

	function get_userdata( $user_id ) {

		if ( ! is_numeric( $user_id ) ) {
			return false;
		}

		if ( empty( $user_id ) ) {
			return false;
		}

		$user_id = absint( $user_id );
		if ( ! $user_id ) {
			return false;
		}

		/**
		 * Check $user is in cache object
		 */

		$user = wp_cache_get( $user_id, 'users' );

		if ( $user ) {

			return get_user_by( 'id', $user_id );

		} else {

			// Initialize new $user
			$user = new WP_User;
		}

		$visitor = XenWord::getVisitor();

		XenWord_XFUserData::xf_user_data( $visitor, $user, $user_id );

		update_user_caches( $user );

		return get_user_by( 'id', $user_id );
	}

endif;

if ( ! function_exists( 'wp_validate_auth_cookie' ) ) :

	function wp_validate_auth_cookie( $cookie = '', $scheme = '' ) {

		if ( XenWord::getVisitor()->getUserId() != 0 ) {
			return XenWord::getVisitor()->getUserId();
		}

		return false;
	}

endif;

if ( ! function_exists( 'cache_users' ) ) :

	/**
	 * Display an error when trying to view the 'Users' tab on the WP Admin page
	 *
	 * @param $users
	 */

	function cache_users( $users ) {

		$xenword_options = get_option( 'xenword_options' );

		if ( $xenword_options['use_wp_users'] == false ) {
			wp_die( __( 'XenWord мост является активным. Пожалуйста обработать все изменения пользователя через XenForo и не WordPress.' ) );
		}
		return $users;
	}

endif;

// End class-xenword-login-xfusers.php