<?php

/**
 * Filename: class-xenword-thread-creation.php
 *
 * Functions to insert posts, custom posts, and pages into XenForo forums
 *
 * @author      LPH
 * @version     3.0.0.01
 * @package     XenWord
 * @subpackage  includes
 *
 * @since 1.0.0.1
 *        Initial Version
 *
 * @since 1.0.0.9
 *        UPDATE the WordPress database thread_id with ID
 *
 * @since 2.5.0.02
 *        UPDATE node_id in xenword_edit_post()
 *
 * @since 2.5.1.05
 *        Added xenword_transitions function
 *
 * @since 3.0.0.01
 *        Added custom excerpt
 *
 * @since 3.0.0.06
 *        Cleaned up switch to not repeat conditionals
 */

/**
 * This section is for security. Do not modify this part:
 * @ignore
 */

if ( ! defined( 'ABSPATH' ) ) exit;

/**
 * This function sets the forum_id and passes it to xenword_insert_thread
 *
 * @param $post_id
 */
function xenword_new_post( $post_id ) {

	$post               = get_post( $post_id );
	$xenword_options    = get_option( 'xenword_options' );

	if ( $xenword_options['use_default_forum'] == true ) {

		/**
		 * The $forum_id will not be written to the post table until xenword_insert_thread is run.
		 *
		 * @var  $forum_id
		 *
		 */
		$forum_id = $xenword_options['default_forum'];
		xenword_insert_thread( $post, $forum_id );

	} elseif ( XenWord::getThreadIdForPost( $post_id ) == '0' || XenWord::getThreadIdForPost( $post_id ) == '' ) {

		/**
		 * The $forum_id was added by the class-xenword-metabox-forumid.php
		 */
		$forum_id = XenWord::getForumIdForPost( $post_id );
		xenword_insert_thread( $post, $forum_id );

	}
}

/**
 * Insert the WordPress post into the XenForo forum.
 *
 * @since Version 1.0.0.1
 *
 * @param $post
 * @param $forum_id
 *
 * @throws Exception
 * @throws XenForo_Exception
 */
function xenword_insert_thread( $post, $forum_id ) {
	// global $wpdb;

	$xenword_options = get_option( 'xenword_options' );

	$title           = $post->post_title;
	$content         = get_post_field( 'post_content', $post->ID );
	$content_except  = get_post_field( 'post_excerpt', $post->ID );
	$excerpt_length  = $xenword_options['xf_excerpt_length'];
	$date            = mysql2date( 'M j Y', $post->post_date );

	/**
	 * Option to show a large featured image rather than a thumbnail
	 *
	 * @since 3.0.0.07
	 */
	if ( $xenword_options['use_large_featured_image'] == true ) {
		$featured_image = wp_get_attachment_image_src( get_post_thumbnail_id( $post->ID ), 'large' );
	} else {
		$featured_image = wp_get_attachment_image_src( get_post_thumbnail_id( $post->ID ), 'thumbnail' );
	}

	$author_id      = $post->post_author;
	$author         = get_the_author_meta( 'user_nicename', $author_id );

	if (empty( $author )) {
		$visitor = XenWord::getVisitor();
		$author= $visitor['username'];
	}

	/** @var  $forumModel XenForo_Model_Forum */
	$forumModel     = XenForo_Model::create('XenForo_Model_Forum')->getModelFromCache( 'XenForo_Model_Forum' );
	$forum          = $forumModel->getForumById( $forum_id );

	if ( ! $forum ) {
		wp_die(  $forum_id . ' is not a correct forum ID. Please check your XenWord forum_id settings.' );
	}

	$errorPhraseKey = '';
	if ( $forumModel->canPostThreadInForum( $forum, $errorPhraseKey ) ) {

		/** @var  $ThreadWriter XenForo_DataWriter_Discussion_Thread */
		$ThreadWriter = XenForo_DataWriter::create( 'XenForo_DataWriter_Discussion_Thread' );
		$ThreadWriter->set( 'user_id', $author_id );
		$ThreadWriter->set( 'username', $author );
		$ThreadWriter->set( 'post_date', $date );

		if ( $title != '' ) {
			$ThreadWriter->set( 'title', $title );
		} else {

			/**
			 * The user entered no title for their post
			 */

			$ThreadWriter->set( 'title', 'New Blog Post' );
		}

		$ThreadWriter->set( 'node_id', $forum['node_id'] );

		/** @var  $threadModel XenForo_Model_Post */
		$threadModel = XenForo_Model_Post::create('XenForo_Model_Post');
		$ThreadWriter->set( 'discussion_state', $threadModel->getPostInsertMessageState( array(), $forum ) );

		/** determine the post body that we should be inserting */
		$post_contains = $xenword_options['first_post_contains'];

		$body       = '';
		$submission = $xenword_options['submission_text'];
		$continue   = $xenword_options['continue_text'];

		if ( $xenword_options['use_submission_text'] == true ) {

			$body .= $author . ' ' . $submission;

		}

		if ( $xenword_options['use_title_text'] == true ) {

			$body .= "\n\n[URL='" . esc_url( get_permalink( $post->ID ) ) . "']" . $title . "[/url]\n\n";

		}

		if ( $xenword_options['use_featured_image'] == true ) {

			$body .= "[IMG]" . $featured_image[0] . "[/IMG]\n\n";

		}

		switch ( $post_contains ) {

			case ( $post_contains == '1' ) :

				/**
				 * This is code for the full WordPress post to show in XenForo.
				 */

				$body .= XenForo_Helper_String::autoLinkBbCode( XenForo_Html_Renderer_BbCode::renderFromHtml( nl2br( $content ) ) );

				break;

			case ( $post_contains == '2' ) :

				/**
				 * This is code for the WordPress excerpt to show in XenForo
				 */

				$body .= '[quote]' . XenForo_Helper_String::autoLinkBbCode( XenForo_Html_Renderer_BbCode::renderFromHtml( nl2br( $content_except ) )  ) . '[/quote]';

				break;

			case ( $post_contains == '3' ) :

				/**
				 * This is code for a customized length of the excerpt.
				 */

				$body .= XenForo_Helper_String::wholeWordTrim( XenForo_Helper_String::autoLinkBbCode( XenForo_Html_Renderer_BbCode::renderFromHtml( nl2br( $content ) ) ), $excerpt_length );

				break;

		}

		$body .= "\n[URL='" . esc_url( get_permalink( $post->ID ) ) . "']" . $continue . "[/URL]\n";

		// $thread_id = XenWord::getThreadIdForPost( $post->ID );
		// wp_die( __('Post ID is ' . $post->ID ) . '<br />Title is ' . $title . '<br />Thread ID is ' . $thread_id . '<br />Forum ID is ' . $forum_id );

		/** @var  $postWriter XenForo_DataWriter_Discussion */
		$postWriter = $ThreadWriter->getFirstMessageDw();
		$postWriter->set( 'message', $body );
		$postWriter->setExtraData( XenForo_DataWriter_DiscussionMessage::DATA_ATTACHMENT_HASH, '' );

		// get tags and place all tag names into an array

		if ( class_exists( 'XenForo_Model_Tag' ) && ( $xenword_options['use_tag_creation'] == true ) ) {

			$rawTags = get_the_tags($post->ID);
			$tags    = array();

			foreach ($rawTags as $tag) {
				$tags[] = $tag->name;
	        }

			$tagger = null;

			// $forum is set via the ForumThreadPost helper

			/** @var XenForo_Model_Tag $tagModel */
			$tagModel = XenForo_Model::create('XenForo_Model_Tag')->getModelFromCache('XenForo_Model_Tag');
			$tagger = $tagModel->getTagger('thread');
			$tagger->setPermissionsFromContext($forum)->setTags($tags);

			$postWriter->mergeErrors($tagger->getErrors());

		}

		$ThreadWriter->preSave();

		//Zend_Debug::dump($ThreadWriter);

		if ( ! $ThreadWriter->hasErrors() ) {

			$ThreadWriter->save();

			$thread     = $ThreadWriter->getMergedData();
			$thread_id  = $thread['thread_id'];

			if ( isset( $tagger ) ) {
				$tagger->setContent($thread['thread_id'], true)->save();
			}

			update_post_meta( $post->ID, 'thread_id', $thread_id );

			update_post_meta( $post->ID, 'forum_id', $forum_id );

			// wp_die( __('The forum id is' . $forum_id . '<br />The title is' . $title . '<br />and the $forum is' . $forum . '<br />Thread ID' . $thread_id ) );

		} // End no errors

	} else {

		/**
		 * Permissions fail
		 */
		wp_die( __( 'Нет разрешения на Пост в выбранном форуме.' . '<br />Форум id Это' . $forum_id . '<br />The title is' . $title . '<br />и $forum Это' . $forum . 'Пожалуйста Проверьте настройки XenForo.' ) );

	}
}

/**
 * This function updates the XenForo post after WordPress post is updated
 * A thread_id must be written or the post will not be updated.
 *
 * @param $post_id
 *
 * @throws Exception
 * @throws XenForo_Exception
 */
function xenword_edit_post( $post_id ) {
	global $post;

	$xenword_options    = get_option( 'xenword_options' );
	$post               = get_post( $post_id );
	$submission         = $xenword_options['submission_text'];
	$continue           = $xenword_options['continue_text'];

	$thread_id          = XenWord::getThreadIdForPost($post_id);

	if ( ! empty( $thread_id ) ) {

		$title          = $post->post_title;
		$content        = $post->post_content;
		$content_except = get_post_field( 'post_excerpt', $post->ID );
		$excerpt_length = $xenword_options['xf_excerpt_length'];

		/**
		 * Option to show a large featured image rather than a thumbnail
		 *
		 * @since 3.0.0.07
		 */
		if ( $xenword_options['use_large_featured_image'] == true ) {
			$featured_image = wp_get_attachment_image_src( get_post_thumbnail_id( $post->ID ), 'large' );
		} else {
			$featured_image = wp_get_attachment_image_src( get_post_thumbnail_id( $post->ID ), 'thumbnail' );
		}

		$author_id      = $post->post_author;
		$author         = get_the_author_meta( 'user_nicename', $author_id );

		/** @var  $threadModel XenForo_Model_Thread */
		$threadModel        = XenForo_Model::create('XenForo_Model_Thread')->getModelFromCache( 'XenForo_Model_Thread' );
		$thread             = $threadModel->getThreadById( $post->{'thread_id'} );
		$thread_first_id    = $thread['first_post_id'];

		$writer = XenForo_DataWriter::create( 'XenForo_DataWriter_Discussion_Thread' );
		$writer->setExistingData( $thread['thread_id'] );
		$writer->setExistingData( 'user_id', $author_id );
		$writer->setExistingData( 'username', $author );

		if ( $title != '' ) {
			$writer->set( 'title', $title );
		} else {
			/**
			 * The user entered no title for their post
			 */
			$writer->set( 'title', 'Новый Пост блога' );
		}

		// Determine the post body that we should be inserting
		$post_contains = $xenword_options['first_post_contains'];

		$body = '';

		if ( $xenword_options['use_submission_text'] == true ) {

			$body .= $author . ' ' . $submission;

		}

		if ( $xenword_options['use_title_text'] == true ) {

			$body .= "\n\n[URL='" . esc_url( get_permalink( $post->ID ) ) . "']" . $title . "[/url]\n\n";

		}

		if ( $xenword_options['use_featured_image'] == true ) {

			$body .= "[IMG]" . $featured_image[0] . "[/IMG]\n\n";

		}

		switch ( $post_contains ) {

			case ( $post_contains == '1' ) :

				/**
				 * This is code for the full WordPress post to show in XenForo.
				 */

				$body .= XenForo_Helper_String::autoLinkBbCode( XenForo_Html_Renderer_BbCode::renderFromHtml( nl2br( $content ) ) );

				// $body = preg_replace( '#\[nlposts .*\]#', '', $body );

				break;

			case ( $post_contains == '2' ) :

				/**
				 * This is code for the WordPress excerpt to show in XenForo
				 */

				$body .= '[quote]' . XenForo_Helper_String::autoLinkBbCode( XenForo_Html_Renderer_BbCode::renderFromHtml( nl2br( $content_except ) )  ) . '[/quote]';

				break;

			case ( $post_contains == '3' ) :

				/**
				 * This is code for a customized length of the excerpt.
				 */

				$body .= XenForo_Helper_String::wholeWordTrim( XenForo_Helper_String::autoLinkBbCode( XenForo_Html_Renderer_BbCode::renderFromHtml( nl2br( $content ) ) ), $excerpt_length );

				break;
		}

		$body .= "\n[URL='" . esc_url( get_permalink( $post->ID ) ) . "']" . $continue . "[/URL]\n";

		$writer = XenForo_DataWriter::create( 'XenForo_DataWriter_DiscussionMessage_Post' );
		$writer->setExistingData( $thread_first_id );
		$writer->set( 'message', $body );
		$writer->preSave();

		if ( ! $writer->hasErrors() ) {

			$writer->save();

			$node_id    = XenWord::getForumIdForPost( $post->ID );
			$thread     = $writer->getMergedData();

			$writer     = XenForo_DataWriter::create( 'XenForo_DataWriter_Discussion_Thread' );
			$writer->setExistingData( $thread['thread_id'] );
			$writer->set( 'node_id', $node_id );

			//wp_die( __('The title is' . $title . '<br />Thread ID' . $thread['thread_id'] ) );

			update_post_meta($post_id, 'thread_id', $thread['thread_id']);

		}
	}
}

/**
 * the post creation actions
 *
 * TODO: Scheduled posts still do not write properly to the XenForo forum.
 */

if ( $xenword_options['use_thread_creation_post'] == true ) {

	/**
	 * @param $new_status
	 * @param $old_status
	 * @param $post
	 *
	 * @return bool
	 */
	function xenword_transitions( $new_status, $old_status, $post ) {

		$post_id    = $post->ID;
		$post_type  = $post->post_type;

		if ( $post_type == 'post' ) {

			if ( $old_status !== 'publish' && $new_status === 'publish' ) {
				xenword_new_post( $post_id );
			}

			if ( $old_status === 'publish' && $new_status === 'publish' ) {

				if ( XenWord::getThreadIdForPost( $post_id ) == '0' || XenWord::getThreadIdForPost( $post_id ) == '' ) {

					// $thread_id = XenWord::getThreadIdForPost( $post_id );
					// wp_die( __('Post ID is ' . $post_id ) . '<br />Thread ID is ' . $thread_id );
					xenword_new_post( $post_id );

				} else {
					// wp_die( __('Post ID is ' . $post_id ) );
					xenword_edit_post( $post_id );
				}
			}
		}
	}

	add_action(  'transition_post_status',  'xenword_transitions', 10, 3 );
}

/**
 * The page creation action
 */

if ( $xenword_options['use_thread_creation_page'] == true ) {
	add_action( 'publish_page', 'xenword_new_post' );
}

/**
 * The custom post type action
 */

if ( $xenword_options['use_thread_creation_custom'] == true ) {
	$custom_post_types = $xenword_options['custom_post_type'];
	foreach ( $custom_post_types AS $custom_post_type ) {
		add_action( 'publish_' . $custom_post_type, 'xenword_new_post' );
	}
}

// End of class-xenword-thread-creation.php