<?php

/**
 * Filename: index.php
 *
 * XenWord class connects WordPress to XenForo through an absolute path.
 *
 * Use the Global $XF in php script to grab XenForo information
 * or use the class::method, e.g. XenWord::getForumIdForPost()
 *
 * @author      LPH
 * @version     3.0.0.01
 * @package     XenWord
 * @subpackage  includes
 *
 * @since 3.0.0.01
 *        Added conditional around $fileDir
 */

/**
 * This section is for security. Do not modify this part:
 * @ignore
 */

if ( ! defined( 'ABSPATH' ) ) exit;

/**
 * Class XenWord
 */
class XenWord {

	/**
	 * @var $visitor
	 */
	public $visitor;

	/**
	 * cache of options
	 * @var $options
	 */
	public $options;

	/**
	 * models
	 * @var array $models
	 */
	protected $models;

	/**
	 * Unique identifier for your plugin.
	 *
	 * The variable name is used as the text domain when internationalizing strings
	 * of text. Its value should match the Text Domain file header in the main
	 * plugin file.
	 *
	 * @since    1.0.0.4
	 *
	 * @var      string
	 */
	protected $plugin_slug = 'xenword';

	/**
	 * @var null
	 */
	protected static $instance = null;

	/**
	 * @param $xenword_options
	 */
	public function __construct( $xenword_options ) {

		$this->models         = array();
		$this->options        = $xenword_options;
		$this->getXenForo();

	}

	/**
	 *
	 */
	public function get_plugin_slug() {
		return $this->plugin_slug;
	}

	/**
	 * @param $xenword_options
	 *
	 * @return null|XenWord
	 */
	public static function get_instance( $xenword_options) {
		// If the single instance hasn't been set, set it now.
		if ( null == self::$instance ) {
			self::$instance = new self($xenword_options);
		}

		return self::$instance;
	}

	/**
	 * Initialize XenForo from an External location
	 *
	 * @throws Zend_Exception
	 */
	public static function getXenForo() {

		// This is required to make the flash media uploader work; accept cookies through HTTP POST
		if ( isset( $_REQUEST['auth_cookie'] ) ) {
			$_COOKIE['xf_session'] = $_REQUEST['auth_cookie'];
		}

		/**
		 * absolute_path is set in XenWord Settings
		 * Use phpinfo() in XenForo admin to get the absolute path
		 * /admin.php?tools/phpinfo
		 */

		if ( !class_exists( 'XenForo_Autoloader' ) ) {

			/** @var $xenword_options */
			$xenword_options = get_option( 'xenword_options' );

			$startTime = microtime( true );

			/** @var  $fileDir */
			$fileDir = $xenword_options['absolute_path'];

			if ( empty($fileDir) ) {
				$fileDir = '/Applications/MAMP/htdocs/community';
			}

			require_once( $fileDir . '/library/XenForo/Autoloader.php' );
			XenForo_Autoloader::getInstance()->setupAutoloader( $fileDir . '/library' );

			XenForo_Application::initialize( $fileDir . '/library', $fileDir, true, array( 'resetOutputBuffering' => false ) );
			XenForo_Application::set('page_start_time', $startTime );
			XenForo_Application::disablePhpErrorHandler();

			if ( $xenword_options['enable_error_reporting'] == true && isset( $xenword_options['enable_error_reporting'] ) ) {
				error_reporting( E_ALL ^ E_NOTICE ^ E_USER_NOTICE ^ E_WARNING );
			} else {
				error_reporting( 0 );
				@ini_set( 'display_errors', 0 );
			}
		}

		XenForo_Session::startPublicSession();

	}

	/**
	 * @return mixed
	 * @throws Zend_Exception
	 */
	public static function getBoardUrl() {
		return XenForo_Application::get('options')->boardUrl;
	}


	/**
	 * Get an instance of XenForo Visitor
	 *
	 * @return XenForo_Visitor
	 */
	public static function getVisitor() {
		return XenForo_Visitor::getInstance();
	}

	/**
	 * Method to get User
	 *
	 * @param null $id
	 *
	 * @return array
	 * @throws XenForo_Exception
	 */
	public function getUser( $id = null ) {

		// Make sure we are logged in
		if($id === null && !$this->getVisitor()->getUserId()) {
			return array();
		}

		$userId = $id !== null ? $id : $this->getVisitor()->getUserId();

		/** @var XenForo_Model_User getFullUserById */
		return XenForo_Model::create('XenForo_Model_User')->getFullUserById($userId);
	}

	/**
	 * @return int
	 */
	public function isLoggedIn() {
		return $this->getVisitor()->getUserId();
	}

	/**
	 * Method to get the thread_id from the postmeta table
	 *
	 * @since 1.0.0.1
	 *        Initial release
	 *
	 * @since 3.0.0.10
	 *        Changed to postmeta table
	 *
	 * @since 3.0.0.11
	 *        Added conditional to check if $post['thread_id'] != 0
	 * @since 3.0.1.04
	 *        Check existence of older values, use update_post_meta
	 *
	 * @param $post_id
	 *
	 * @return bool
	 */
	public static function getThreadIdForPost( $post_id ) {

		$post = get_post( $post_id, ARRAY_A );

		// Get an array of thread_ids
		$thread_ids = get_post_custom_values( 'thread_id', $post_id );

		if ( is_array( $thread_ids ) ) {

			foreach ( $thread_ids as $thread_id ) {
				if ( ! empty( $thread_id ) ) {

					// Return the thread_id
					return $thread_id;

				} elseif ( isset( $post['thread_id'] ) && $post['thread_id'] != '0' ) {

					// Update thread_id to meta
					update_post_meta( $post->ID, 'thread_id', $post['thread_id'] );
					return $post['thread_id'];

				} else {
					return false;
				}
			}
		}
	}

	/**
	 * Method to pull the forum_id from postmeta
	 *
	 * @since 3.0.0.10
	 *        Changed to postmeta table
	 * @since 3.0.1.04
	 *        Check existence of older values, use update_post_meta
	 *
	 * @param $post_id
	 *
	 * @return bool
	 */
	public static function getForumIdForPost( $post_id ) {

		// Get an array of forum_ids
		$forum_ids = get_post_custom_values( 'forum_id', $post_id );

		if ( is_array( $forum_ids ) ) {

			foreach ( $forum_ids as $forum_id ) {
				if ( ! empty( $forum_id ) ) {

					// Return the forum_id
					return $forum_id;

				} elseif ( isset( $post['forum_id'] ) && $post['forum_id'] != '0' ) {

					// Update forum_id to meta
					update_post_meta( $post->ID, 'thread_id', $post['forum_id'] );
					return $post['forum_id'];

				} else {
					return false;

				}
			}
		}
	}

	/**
	 * Method to get the XenForo Resource ID
	 *
	 * @param $post_id
	 *
	 * @return bool
	 */
	public static function getResourceIdForPost( $post_id ) {

		$resource_ids = get_post_custom_values( 'resource_id', $post_id );

		foreach ( $resource_ids as $resource_id) {
			if ( $resource_id != 0 || $resource_id != '' ) {
				return $resource_id;
			} else {
				return false;
			}
		}
	}

	/**
	 * Method to get the XenForo Resource Category ID
	 *
	 * @param $post_id
	 *
	 * @return bool
	 */
	public static function getResourceCatIdForPost( $post_id ) {

		$resource_category_ids = get_post_custom_values( 'resource_category_id', $post_id );

		foreach ( $resource_category_ids as $resource_category_id ) {
			if ( $resource_category_id != 0 || $resource_category_id != '' ) {
				return $resource_category_id;
			} else {
				return false;
			}
		}
	}

	/**
	 * Method for the XenForo logout
	 *
	 * @return bool
	 */
	public static function getLogout() {

		// remove an admin session if we're logged in as the same person
		if (XenForo_Visitor::getInstance()->get('is_admin'))
		{
			$adminSession = new XenForo_Session( array('admin' => true) );
			$adminSession->start();
			if ( $adminSession->get('user_id') == XenForo_Visitor::getUserId() )
			{
				$adminSession->delete();
			}
		}

		/** @var XenForo_Model_Session */
		XenForo_Model::create('XenForo_Model_Session')->processLastActivityUpdateForLogOut( XenForo_Visitor::getUserId() );

		XenForo_Application::get('session')->delete();
		XenForo_Helper_Cookie::deleteAllCookies(
			array('session'),
			array('user' => array( 'httpOnly' => false) )
		);

		XenForo_Visitor::setup(0);

		return true;

	}

	/**
	 * Method for Admin Logout
	 */
	public static function getAdminLogout() {
		if ( XenForo_Visitor::getInstance()->get( 'is_admin' ) ) {
			$adminSession = new XenForo_Session( array( 'admin' => true ) );
			$adminSession->start();
			if ( $adminSession->get( 'user_id' ) == XenForo_Visitor::getUserId() ) {
				$adminSession->delete();
			}
		}
	}

	/**
	 * Method to get the XenForo Forum by ID
	 *
	 * @param       $id
	 * @param array $fetchOptions
	 *
	 * @return mixed
	 * @throws XenForo_Exception
	 */
	public function getForumById( $id, $fetchOptions=array() ) {
		/** @var $forumModel XenForo_Model_Forum getForumById */
		$forumModel = XenForo_Model::create('XenForo_Model_Forum');
		return $forumModel->getForumById( $id, $fetchOptions );
	}

	/**
	 * @param       $ids
	 * @param array $fetchOptions
	 *
	 * @return array
	 * @throws XenForo_Exception
	 */
	public function getForumsByIds( $ids, $fetchOptions=array() ) {
		/** @var $forumModel XenForo_Model_Forum getForumByIds */
		$forumModel = XenForo_Model::create('XenForo_Model_Forum');
		return $forumModel->getForumsByIds($ids, $fetchOptions);
	}

	public function getForums(array $conditions = array(), array $fetchOptions = array()) {
		return XenForo_Model::create('XenForo_Model_Forum')->getForums($conditions, $fetchOptions);
	}

	public function getThreadsByIds($ids, $fetchOptions=array()) {
		return XenForo_Model::create('XenForo_Model_Thread')->getThreadsByIds($ids, $fetchOptions);
	}

	public function getThreadById($id, $fetchOptions=array()) {
		return XenForo_Model::create('XenForo_Model_Thread')->getThreadById($id, $fetchOptions);
	}

	public function getThreads(array $conditions, array $fetchOptions = array()) {
		return XenForo_Model::create('XenForo_Model_Thread')->getThreads($conditions, $fetchOptions);
	}

	public function renderTemplate($name, $params=array(), $styleId=null, $languageId=null) {
		// user
		$user = $this->getUser();
		// Template
		$template = new XenForo_Template_Public($name, $params);
		$template->setStyleId(($styleId!==null ? $styleId : $user['style_id']));
		$template->setLanguageId(($languageId!==null ? $languageId : $user['language_id']));
		return $template->render();
	}

	public function renderAdminTemplate($name, $params=array(), $styleId=null, $languageId=null) {
		// user
		$user = $this->getUser();
		// Template
		$template = new XenForo_Template_Admin($name, $params);
		$template->setStyleId(($styleId!==null ? $styleId : $user['style_id']));
		$template->setLanguageId(($languageId!==null ? $languageId : $user['language_id']));
		return $template->render();
	}
}

$XF = new XenWord( $xenword_options );

// End index.php