<?php

/**
 * Filename: Comments.php
 *
 * Grabs the WordPress comments and writes the comments to the XenForo xf_post table.
 *
 * @author      LPH
 * @version     3.0.1.01
 * @package     XenWord
 * @subpackage  /XenForo/library/XenWord
 *
 * @since 3.0.1.01
 *        Initial release
 */

class XenWord_Comments {

	/**
	 * comments
	 * @var array
	 */
	protected $comments;

	/**
	 * Number of comments
	 * @var array
	 */
	protected $commentnumbers;

	public function __construct( $xenword_options ) {

		$this->comments       = array();
		$this->commentnumbers = array();
		$this->options        = $xenword_options;

		$this->hooks();
	}

	public function hooks() {
		add_filter( 'get_comments_number', array( &$this, 'xf_comment_number' ), 10, 2 );
		add_filter( 'comments_array', array( &$this, 'BuildCommentsArray' ), 10, 2 );
		add_filter( 'comment_text', array( &$this, 'embed_BBCode' ), 10, 1 );
	}

	/**
	 * Check if we should use XenForo to handle the comment system, or leave it to the WP default
	 *
	 * @param string $comment_array
	 *
	 * @return bool
	 */
	public static function useXenForoCommentHandler( $comment_array = '' ) {
		global $wp_query;

		// Saving a comment
		if ( $comment_array != '' ) {

			if ( is_array( $comment_array ) ) {
				$post_id = $comment_array['comment_post_ID'];
			} else {

				// Object of StdClass

				/** @var  $post_id */
				$post_id = $comment_array->{'comment_post_ID'};
			}

			if ( XenWord::getThreadIdForPost( $post_id ) != false ) {
				return true;
			} else {
				return false;
			}

		} elseif ( isset( $wp_query->post->ID ) ) {
			if ( XenWord::getThreadIdForPost( $wp_query->post->ID ) != '0' || XenWord::getThreadIdForPost( $wp_query->post->ID ) != '' ) {
				return true;
			} else {
				return false;
			}
		}

		return false;
	}

	/**
	 * @param $post_id
	 *
	 * @return mixed
	 * @throws XenForo_Exception
	 */
	public static function getCommentNumbersFromCache( $post_id ) {
		global $xw_comments;

		if ( isset( $xw_comments->commentnumbers[ $post_id ] ) ) {
			return $xw_comments->commentnumbers[ $post_id ];
		} else {
			global $wp_query;
			$post_arr = $wp_query->posts;

			if ( count( $post_arr ) == 1 ) {
				$the_comments = self::getCommentsFromCache( $post_id );
				$xw_comments->commentnumbers[ $post_id ] = count( $the_comments ) - 1;
			} else {
				$thread_ids = array();

				foreach ( $post_arr as $the_post ) {
					// We don't have a thread for this post
					if ( $the_post->thread_id == '0' ) {
						$xw_comments->commentnumbers[ $the_post->ID ] = 0;
					} else {
						$thread_ids[] = (int) $the_post->thread_id;
					}
				}

				/** @var  $threadModel XenForo_Model_Thread */
				$threadModel = XenForo_Model::create('XenForo_Model_Thread')->getModelFromCache( 'XenForo_Model_Thread' );
				$threads     = $threadModel->getThreadsByIds( $thread_ids );

				foreach ( $post_arr as $the_post ) {
					// We sent a request for a thread
					if ( $the_post->thread_id != '0' ) {
						$xw_comments->commentnumbers[ $the_post->ID ] = $threads[ (int) $the_post->thread_id ]['reply_count'];
					}
				}
			}

			return $xw_comments->commentnumbers[ $post_id ];
		}
	}

	/**
	 * @param $post_id
	 *
	 * @return mixed
	 * @throws XenForo_Exception
	 */
	public function getCommentsFromCache( $post_id ) {
		global $xw_comments;
		if ( isset( $xw_comments->comments[ $post_id ] ) ) {
			return $xw_comments->comments[ $post_id ];
		} else {

			/** @var  $postModel XenForo_Model_Post */
			$postModel = XenForo_Model::create('XenForo_Model_Post')->getModelFromCache( 'XenForo_Model_Post' );
			$post      = get_post( $post_id );

			$postFetchOptions = array(
				'deleted'    => false,
				'moderated'  => false,
				'join'       => XenForo_Model_Post::FETCH_USER,
				'likeUserId' => XenForo_Visitor::getInstance()->get('user_id')
			);

			$xw_comments->comments[ $post_id ] = $postModel->getPostsInThread( XenWord::getThreadIdForPost($post_id), $postFetchOptions );

			return $xw_comments->comments[ $post_id ];
		}
	}

	/**
	 * Pull a count from the XenForo side
	 * This count will not include older comments before linking post to a thread
	 *
	 * @param $count
	 * @param $post_id
	 *
	 * @return mixed
	 */
	public static function xf_comment_number( $count, $post_id ) {

		if ( self::useXenForoCommentHandler() ) {

			return self::getCommentNumbersFromCache( $post_id );

		} else {

			return $count;
		}
	}

	/**
	 * Create an array of replies that have not been written to the comments table
	 * then write these comments to the comment and commentmeta table.
	 *
	 * Separate out comments that have been written to the comment meta table
	 *
	 * @param $comment
	 * @param $post_id
	 *
	 * @return array|string
	 * @throws Zend_Exception
	 */
	public function BuildCommentsArray( $comment, $post_id ) {
		global $wp_query;

		/**
		 * Update commentmeta information from older XenWord releases using XF_post_ID?
		 *
		 * Very important for backwards compatibility
		 *
		 */
		$args = array( 'post_id' => $post_id, );
		$comments = get_comments($args);

		foreach( $comments as $comment ) {
			if ( $comment->XF_post_ID != '0' ) {
				update_comment_meta( $comment->comment_ID, 'post_id', $comment->XF_post_ID );
			}
		}

		/**
		 * Loads the XenWord options saved in the XenWord panel.
		 * @var  $xenword_options
		 */
		$xenword_options = get_option( 'xenword_options' );

		/** Are we displaying comments in WP? */
		if ( $xenword_options['display_comments_in_wp'] == true ) {

			// Set the number of comments per page
			$wp_query->set( 'comments_per_page', $xenword_options['comments_per_page'] );


			/**
			 * Get comments and build an array with all comments from XenForo side
			 */

			// $formatter = XenForo_BbCode_Formatter_Base::create();
			$parser = new XenForo_BbCode_Parser( new XenForo_BbCode_Formatter_Wysiwyg );

			// $bbCodeParser = new XenForo_BbCode_Parser(new XenForo_BbCode_Formatter_Wysiwyg);

			$comments = self::getCommentsFromCache( $post_id );

			/**  load timezone related options */
			$gmt_offset  = (float) get_option( 'gmt_offset' ) * 3600;

			// The earliest XF comment
			$earliest_timestamp = 999999999999999;

			$mycomments = '';

			if ( isset( $mycomments ) ) {

				$i = - 1;
				foreach ( $comments as $mycomment ) {
					if ( $i != - 1 ) {
						$mycomments[ $i ]                       = new StdClass;
						$mycomments[ $i ]->comment_ID           = (string) $mycomment['post_id'];
						$mycomments[ $i ]->comment_post_ID      = (string) $post_id;
						$mycomments[ $i ]->comment_author       = $mycomment['username'];
						$mycomments[ $i ]->comment_author_email = $mycomment['email'];

						if ( $mycomment['user_id'] != '0' ) {
							$mycomments[ $i ]->comment_author_url = XenForo_Link::buildPublicLink( 'canonical:members/' . strtolower( $mycomment['username'] ) . '.' . $mycomment['user_id'] . '/' );
						} else {
							$mycomments[ $i ]->comment_author_url = '';
						}

						$mycomments[ $i ]->comment_date     = gmdate( 'Y-m-d H:i:s', $mycomment['post_date'] + $gmt_offset );
						$mycomments[ $i ]->comment_date_gmt = gmdate( 'Y-m-d H:i:s', $mycomment['post_date'] );
						$mycomments[ $i ]->comment_content  = $parser->render( $mycomment['message'] );

						$mycomments[ $i ]->comment_content = str_replace("styles", XenWord::getBoardUrl() . "/styles", $mycomments[ $i ]->comment_content);

						$mycomments[ $i ]->comment_karma    = $mycomment['likes'];
						$mycomments[ $i ]->comment_approved = '1';
						$mycomments[ $i ]->comment_type     = '';
						$mycomments[ $i ]->comment_parent   = '0';
						$mycomments[ $i ]->user_id          = $mycomment['user_id'];
						$mycomments[ $i ]->timestamp        = strtotime( $mycomments[ $i ]->comment_date_gmt );

						if ( $mycomments[ $i ]->timestamp < $earliest_timestamp ) {
							$earliest_timestamp = $mycomments[ $i ]->timestamp;
						}
					}
					$i ++;
				}
			}


			/**
			 * Get comments already in the comment meta
			 * return $in_comment_meta array
			 */

			// Get WordPress post_id for Post
			$wp_post = get_post( $post_id );

			// Get XenForo thread_id based on WP post_id
			$threadId = get_post_meta( $wp_post->ID, 'thread_id', true );

			// Get all posts based on thread_id

			/** @var  $postdModel XenForo_Model_Thread */
			$postModel = XenForo_Model::create( 'XenForo_Model_Post' )->getModelFromCache( 'XenForo_Model_Post' );

			$fetchOptions = array(
				'order' => 'post_date',
				'orderDirection' => 'desc',
				'discussion_state' => 'visible',
			);

			/** @var $postModel XenForo_Model_Post $posts */
			$postIds = $postModel->getPostsInThread( $threadId, $fetchOptions );

			// Array shift
			array_shift($postIds);

			// Show the correct $postIds array which includes post_id, username, message
			// var_dump($postIds);

			foreach ( $postIds as $postId ) {
				// Zend_Debug::dump($postId['post_id'])

				// Properly shows the post_id for each post belonging to the thread_id
				// echo $postId['post_id'] . '<br />';

				foreach ( $comment as $in_comment_meta ) {

					// Zend_Debug::dump($in_comment_meta);

					//echo $in_comment_meta->comment_ID;

					$meta_postId = get_comment_meta( $in_comment_meta->comment_ID, 'post_id', true );

					// echo $meta_postId . '<br />';

					// Zend_Debug::dump($meta_postId);

					// If meta post_id from comment meta is in array $postIds
					if ( $postId['post_id'] != $meta_postId ) {

						// echo $postId['post_id'] . '<br />';

						// Zend_Debug::dump($postId['post_id']);

					}

				}
			}

			/**
			 * Check if we found any comments we need to remove, and if so remove
			 */

			/** @var $removecomments */
			if ( $removecomments != array() ) {

				if ( is_array( $removecomments ) ) {
					$mycomments = array_diff( $mycomments, $removecomments);
				} else {
					$mycomments = $removecomments;
				}

			}

			/**
			 * Remove  comments already in the comments meta using array_diff
			 */

			/**
			 * Add it to $wp_query so basic wp functions will use the correct comment array,
			 * set up pagination stuff
			 */
			$wp_query->comments = $mycomments;

			return $mycomments;


		} //return $comment;
	}

	/**
	 * Replaces BBCode tags as HTML
	 *
	 * @param $content
	 *
	 * @return mixed
	 * @throws Zend_Exception
	 *
	 * @since 2.5.0.05
	 *        Replaced the /e modifier
	 * @since 2.5.1.01
	 *        Added BBCode for user, spoiler, YouTube media
	 * @since 2.5.1.04
	 *        Added er_what BBCode
	 */
	public function embed_BBCode($content) {

		$content = preg_replace('~\[img\](.+?)\[\/img\]~i', '<img src=\"$1\" alt=\"" . basename("$1") . "\" />', $content);

		$content = preg_replace('~\[user=(.*?)\](.*?)\[/user\]~i', '<a href=' . XenWord::getBoardUrl() . '/members/$1>' . '$2</a>', $content);

		$content = preg_replace('~\[/user\]~i', '', $content);

		$content = preg_replace('~\[attach=.*?\](.*?)\[/attach\](?!\[/attach\])~i', '<img src=' . XenWord::getBoardUrl() . '/attachments/$1 />', $content);

		$content = preg_replace('~\[/attach\]~i', '', $content);

		$content = preg_replace('~\[media=(.*?)\](.*?)\[/media\](?!\[/media\])~i', '<embed src="http://www.youtube.com/v/$2" width="415" height="250"></embed>', $content);

		$content = preg_replace('~\[/media\]~i', '', $content);

		$content = preg_replace('~\[b\](.+?)\[\/b\]~i', '<strong>$1</strong>', $content);

		$content = preg_replace('~\[i\](.+?)\[\/i\]~i', '<em>$1</em>', $content);

		$content = preg_replace('~\[code\](.+?)\[\/code\]~i', '<code>$1</code>', $content);

		$content = preg_replace('~\[php\](.+?)\[\/php\]~i', '<pre>$1</pre>', $content);

		$content = preg_replace('~\[quote\]~siU', '<blockquote>', $content);

		$content = preg_replace('~\[quote=(.*)\]~siU', '<blockquote>', $content);

		$content = preg_replace('~\[/quote\]~siU', '</blockquote>', $content);

		$content = preg_replace('~\[spoiler\](.*?)\[/spoiler\](?!\[/spoiler\])~i', '<strong>SPOILER</strong><br />$1<br /><br />', $content);

		$content = preg_replace('~\[spoiler=(.*?)\](.*?)\[/spoiler\](?!\[/spoiler\])~i', '<strong>SPOILER</strong><br />$2<br /><br />', $content);

		$content = preg_replace('~\[/spoiler\]~i', '', $content);

		$content = preg_replace('~\:confused\:~i', ':???:', $content);

		$content = preg_replace('~\:rolleyes\:~i', ':roll:', $content);

		$content = preg_replace('~\:p~i', ':razz:', $content);

		$content = preg_replace('~o_O~i', ':?:', $content);

		return $content;
	}
}

$xw_comments = new XenWord_Comments( $xenword_options );